#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <assert.h>

#include "e1.h"
#include "lspec.h"

#ifdef RCSID
static char rcsid[] = "$Id: do_doc.c,v 1.5 1995/11/18 22:11:42 root Exp $";
#endif /* RCSID */

#ifdef COPYRIGHT

/*
 * Copyright (c), 1995, compuPage, Inc.
 * All Rights Reserved, Cal Page, <root@lorax.mv.com>
 *
 * This software comes with ABSOLUTELY NO WARRANTY.
 *
 * This software is released under the GPL, see
 * COPYING in source directory for further details.
 */

#endif /* COPYRIGHT */

/*
 * doc ( doc-file , src-filespec-list [ , lib-list ] )
 *
 *    next -> next -> next
 *    | | |
 *    | | - name -> doc-file
 *    | |-- auxRoot -> src-filespec-list
 *    |---- libRoot -> lib-list
 *
 */
int do_doc( void )
{
    int tok;
    char *filespec;
    LSPECPTR tmp;
    char **value;
    int i;
    int lib_list_flag = 1;

    /* add 'doc' to 'all' */
    lspec_add(lspec_new("doc"),&all_root);

    /* skip '(' */
    tok = yylex();
    if ( tok != E1_TOK_brace )
      {
	  printf("do_doc: missing '('\n");
	  exit(1);
      }

    /* get filespec */
    tok = yylex ();
    if ( tok != E1_TOK_filespec )
      {
	  printf("do_doc: missing or bad filespec = <%s>\n",
		 yytext);
	  exit(1);
      }
    filespec = strdup(YYTEXT);

    /* add to all_root */
    tmp = lspec_add(lspec_new(filespec),&doc_root);

    tok = yylex ();
    if ( tok != E1_TOK_comma )
      {
	  printf("do_doc: missing comma <%s>\n",
		 yytext);
	  exit(1);
      }

    /* get src file list */
    while ( 1 )
      {
	  tok = yylex();

	  /* are we done ??? */
	  if ( tok == E1_TOK_brace_close )
	    {
		lib_list_flag = 0;
		break;
	    }
	  /* are we done ??? */
	  if ( tok == E1_TOK_comma )
	    {
		break;
	    }

	  if ( tok != E1_TOK_filespec )
	    {
		printf("do_doc: missing or bad filespec = <%s>\n",
		       yytext);
		exit(1);
	    }

	  /* remove any that are necessary */
	  if ( remlister ( YYTEXT , &tmp->auxNext ) )
	    continue;

	  if ( !isBuildable(YYTEXT) )
	    /* don't glob things we can't build */
	    lspec_add(lspec_new(YYTEXT),&tmp->auxNext);
	  else
	    {
		value = glob_filename ( YYTEXT );
		if ( value == NULL )
		  {
		      printf("out of memory\n");
		      exit(1);
		  }
		else if ((int) value == -1)
		  perror ( YYTEXT );
		else
		  for (i = 0; value[i] != NULL; i++)
		    {
			lspec_add(lspec_new(value[i]),&tmp->auxNext);
		    }
	    }
      }

    /* now get liblist */
    while ( lib_list_flag )
      {
	  tok = yylex();

	  /* are we done ??? */
	  if ( tok == E1_TOK_brace_close )
	    break;

	  if ( tok != E1_TOK_filespec )
	    {
		printf("do_doc: missing or bad filespec = <%s>\n",
		       yytext);
		exit(1);
	    }

	  /* remove any that are necessary */
	  if ( remlister ( YYTEXT , &tmp->libNext ) )
	    continue;

	  if ( strchr(YYTEXT,'/') != NULL ||
	       YYTEXT[0] == '-' )
	    {
		lspec_add(lspec_new(YYTEXT),&tmp->libNext);
	    }
	  else
	    {
		value = glob_filename ( YYTEXT );
		if ( value == NULL )
		  {
		      printf("out of memory\n");
		      exit(1);
		  }
		else if ((int) value == -1)
		  perror ( yytext );
		else
		  for (i = 0; value[i] != NULL; i++)
		    {
			lspec_add(lspec_new(value[i]),&tmp->libNext);
		    }
	    }
      }

    free(filespec);
    return 1;
}
