/* e1.l */


 /* the "incl" state is used for picking up the name
  * of an include file
  */
%x incl

%{
#include <stdio.h>
#include "e1.h"

#ifdef RCSID
static char rcsid[] = "$Id: e1.l,v 1.13 1995/11/18 22:13:23 root Exp $";
#endif /* RCSID */

#ifdef COPYRIGHT

/*
 * Copyright (c), 1995, compuPage, Inc.
 * All Rights Reserved, Cal Page, <root@lorax.mv.com>
 *
 * This software comes with ABSOLUTELY NO WARRANTY.
 *
 * This software is released under the GPL, see
 * COPYING in source directory for further details.
 */

#endif /* COPYRIGHT */

int line_num = 1;
int comment_caller;
int old_state;

/* code for include files */
#define MAX_INCLUDE_DEPTH 12
YY_BUFFER_STATE include_stack[MAX_INCLUDE_DEPTH];
int include_stack_ptr = 0;

%}

DIGIT [0-9]

XDIGIT [0-9A-Fa-f]

ID    [a-zA-Z][a-zA-Z0-9_]*

CFCT  [a-zA-Z][a-zA-Z0-9_]*

FILESPEC  [\$\~\/\-\*a-zA-Z0-9_.]*

   /* used to 'eat' C comments */
%x comment

   /* used when in 'exe', 'obj', or 'lib' */
%x inverb

  /* used when processing '$' variables */
%x indollar
%x indollar1

%%

include             BEGIN(incl);

<incl>[ \t]*      /* eat the whitespace */
<incl>[^ \t\n]+   { /* got the include file name */
         if ( include_stack_ptr >= MAX_INCLUDE_DEPTH )
             {
             fprintf( stderr, "Includes nested too deeply" );
             exit( 1 );
             }

         include_stack[include_stack_ptr++] = YY_CURRENT_BUFFER;
         yyin = fopen( yytext, "r" );
         if ( ! yyin )
             {
             printf("can't open include file = <%s>\n",yytext);
             exit(0);
             }

          yy_switch_to_buffer(
             yy_create_buffer( yyin, YY_BUF_SIZE ) );
          BEGIN(INITIAL);
         }

<<EOF>> {
         if ( --include_stack_ptr < 0 )
             {
             yyterminate();
             }
          else
             {
             yy_delete_buffer( YY_CURRENT_BUFFER );
             yy_switch_to_buffer(
                  include_stack[include_stack_ptr] );
             }
         }


"/*"    {
         comment_caller = YY_START;
         BEGIN(comment);
         }
<comment>[^*\n]*        /* eat anything that's not a '*' */
<comment>"*"+[^*/\n]*   /* eat up '*'s not followd by '/'s */
<comment>\n ++line_num; /* eat \n's in comment             */
<comment>"*"+"/" {
                 BEGIN(comment_caller);
                 }

[A-Z]+ {
        old_state = YY_START;
        BEGIN(indollar);
        return E1_TOK_dollar;
        }

<indollar>[ \t]+  /* eat all space/tab in 'indollar' mode */

<indollar1>[ \t]+  /* eat all space/tab in 'indollar' mode */

<indollar>\= {
             BEGIN(indollar1);
             return E1_TOK_dollar_equal;
             }

<indollar1>.* {
             BEGIN(old_state);
             return E1_TOK_dollar_rvalue;
             }

install {
     old_state = YY_START;
     BEGIN(inverb);
     return E1_TOK_install;
     }

obj {
     old_state = YY_START;
     BEGIN(inverb);
     return E1_TOK_obj;
     }

lib {
     old_state = YY_START;
     BEGIN(inverb);
     return E1_TOK_lib;
     }

exe {
     old_state = YY_START;
     BEGIN(inverb);
     return E1_TOK_exe;
     }

man {
     old_state = YY_START;
     BEGIN(inverb);
     return E1_TOK_man;
     }

doc {
     old_state = YY_START;
     BEGIN(inverb);
     return E1_TOK_doc;
     }

kit {
     old_state = YY_START;
     BEGIN(inverb);
     return E1_TOK_kit;
     }

move {
     old_state = YY_START;
     BEGIN(inverb);
     return E1_TOK_move;
     }

softlink {
     old_state = YY_START;
     BEGIN(inverb);
     return E1_TOK_softlink;
     }

platform-linux return E1_TOK_platform_linux;

platform-linux-elf return E1_TOK_platform_linux;

<inverb>\, return E1_TOK_comma;

<inverb>\( return E1_TOK_brace;

<inverb>{FILESPEC} return E1_TOK_filespec;

<inverb>\) {
           BEGIN(old_state);
           return E1_TOK_brace_close;
           }

\-.* return E1_TOK_minus;

<inverb>\".*\" return E1_TOK_string;

<inverb>[ \t]+ /* eat all whitespace */

[ \t]+ /* eat all whitespace */

<indollar,inverb>\n     ++line_num;

\n     ++line_num;

%%

